/*
	fnhex.c	     	11/20/86

	% hex_funcs

	Hexadecimal editting functions.
	Just like int_funcs except no minus sign.

	C-scape 3.1
	Copyright (c) 1986, 1987, 1988 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	10/01/87 jmd 	added casting
	 4/06/88 jmd 	added call to sed_DoSpecial
	 5/12/88 jmd	added calls to sed_GetScratchPad()
	 9/17/88 jmd 	added std_ funcs
	10/09/88 jmd 	added SED_ABORT support
	10/14/88 jdc	added var_size element to field_funcs_struct

	 6/07/89 jmd	added test for mouse code (later removed)
	07/20/89 jdc	changed hex_senter, sexit calls in hex_fexit to
					sed_DoFieldSenter, sed_DoFieldSexit
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "cscape.h"
#include "fnfunc.h"			/* for field functions */
#include "strdecl.h"		/* for C-scape string functions */
#include "scancode.h"

OGLOBAL field_funcs_struct hex_funcs = {
	num_fenter,
	hex_fexit,
	hex_fkey,
	hex_senter,
	hex_sexit,
	sizeof(int)
};

boolean hex_fexit(sed)
	sed_type sed;
{
	if (sed_GetBaton(sed) != SED_ABORT) {
		sed_DoFieldSexit(sed, sed_GetFieldNo(sed));
		sed_DoFieldSenter(sed, sed_GetFieldNo(sed));
		sed_UpdateCurrField(sed);
	}

	return(std_fexit(sed));
}

void hex_fkey(sed)
	sed_type sed;
{
	int   scancode, key;

	scancode = kb_Read();

	if (sed_DoSpecial(sed, scancode))
		return;
	if (special_key(sed, scancode))
		return;
	if (inter_field(sed, scancode))
		return;
	if (inter_page(sed, scancode))
		return;

	switch(scancode) {
	case BACKSPACE:
		sed_PullLeft(sed);
		strcpy(sed_GetScratchPad(sed), sed_GetCurrRecord(sed));
		sed_SetCurrRecord(sed, strhex(sed_GetScratchPad(sed)));
		sed_UpdateCurrField(sed);
		break;
	default:
		/* 	Overwrite leading zeroes. */
		key = ascii(scancode);
		if (isxdigit(key) && sed_GetChar(sed, 0) == '0') {
			sed_PushLeft(sed, key);
			strcpy(sed_GetScratchPad(sed), sed_GetCurrRecord(sed));
			sed_SetCurrRecord(sed, strhex(sed_GetScratchPad(sed)));
			sed_UpdateCurrField(sed);
		}
		break;
	}

	/* reset baton */
	sed_SetBaton(sed, -1);
}

void hex_senter(sed, fieldno)
	sed_type sed;
	int fieldno;
/*
	Convert native type to string for record.
*/
{
	sprintf(sed_GetScratchPad(sed), "%x", *((int *) sed_GetVar(sed, fieldno)));

	strright(sed_GetScratchPad(sed), sed_GetRecordLen(sed, fieldno));
	sed_SetRecord(sed, strhex(sed_GetScratchPad(sed)), fieldno);

	std_senter(sed, fieldno);
}

void hex_sexit(sed, fieldno)
	sed_type sed;
	int fieldno;
/*
	Converts record back to native type.
*/
{
	if (sed_GetBaton(sed) != SED_ABORT) {
		sscanf(sed_GetRecord(sed, fieldno), "%x", (int *) sed_GetVar(sed, fieldno));
	}
}

char *strhex(s)
	char *s;
/*
	Formats a hexadecimal string
	The string returned is a right justified hexadecimal
	number padded out with leading zeroes.
*/
{
	char *p;

	for (p = s + strlen(s) - 1; p >= s; p--) {
		if (*p == ' ')
			*p = '0';
	}

	return(s);
}


