/*
	fnlist.c     	10/02/87

	% list_funcs

	List Functions:
	A field has multiple choices.
	A popup menu displays the choices.
	The Variable is a char *
	and contains the selected choice.

	The choices are contained in the field's second data pointer in
	the form "choice 1, choice 2, choice 3"  (commas delimit).

	The cursor is turned off.

	C-scape 3.1
	Copyright (c) 1987, 1988 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 3/31/88 jmd	Removed tinkering of static strings
	 4/06/88 jmd 	added call to sed_DoSpecial
	 4/13/88 jmd 	changed MAX_FIELD_LEN to MAX_CHOICE_LEN
	 9/15/88 jmd 	removed vid_Cursor calls
	 9/17/88 jmd 	added std_ funcs
	10/09/88 jmd 	added SED_ABORT support
	10/14/88 jdc	added var_size element to field_funcs_struct
	11/05/88 jmd	removed menu_Close
	12/14/88 jdc	remove pointer subtraction from list_GetChoice()
	12/20/88 jmd	tests for missing list elements

	 2/07/89 jmd	added char * cast
	 4/10/89 jmd	added oakpriv.h
	 6/07/89 jmd	added test for mouse code (later removed)
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "cscape.h"
#include "fnfunc.h"			/* for field functions */
#include "scancode.h"

#include "oakpriv.h"		/* for memmove() macro */

#define	DELIMITER	','


char *list_GetChoice(_arg3(char *, int, char *));

#define MAX_CHOICE_LEN	80			/* maximum length of a choice */

OGLOBAL field_funcs_struct list_funcs = {
	stdNoCur_fenter,
	std_fexit,
    list_fkey,
	string_senter,
	string_sexit,
	VAR_STRING
};

void list_fkey(sed)
	sed_type sed;
/*
	Cycle through list of changes when the SPACE bar is pressed.
*/
{
	int 		scancode;
	menu_type 	list_menu;
	sed_type 	list_sed;
	int 		fieldno, row, col;
	char 	   *list, s[MAX_CHOICE_LEN + 1];
	byte 		reg, sel;
	int i = 0;

	scancode = kb_Read();

	if (sed_DoSpecial(sed, scancode))
		return;
	if (special_key(sed, scancode))
		return;
	if (inter_field(sed, scancode))
		return;
	if (inter_page(sed, scancode))
		return;


	fieldno = sed_GetFieldNo(sed);

	if ((list = (char *) sed_GetCurrFieldData(sed, 1)) != NULL) {

		list_menu = menu_Open();

		/* set the color of the list */
		sed_GetFieldColors(sed, fieldno, &reg, &sel);

		while (list_GetChoice(list, i, s) != NULL) {
			menu_Printf(list_menu, "@p[%d,0] @f[%s] ", i++, NULL, &menu_funcs, s);
		}

		menu_Flush(list_menu);

		list_sed = sed_Open(list_menu);
		sed_SetColors(list_sed, sel, sel, reg);
		sed_SetBorder(list_sed, bd_1);

		/* adjust the position of the list */
		row = sed_GetFieldRow(sed, fieldno) - sed_GetBorderHeight(list_sed)/2;
		row = (row < disp_GetHeight() - sed_GetBorderHeight(list_sed) - 1) ? row :
				disp_GetHeight() - sed_GetBorderHeight(list_sed) - 2;
		row = (row > 0) ? row : 0;
		col = sed_GetFieldLastCol(sed, fieldno);
		col = (col + sed_GetBorderWidth(list_sed) < disp_GetWidth()) ? col :
				disp_GetWidth() - sed_GetBorderWidth(list_sed) - 2;

		if (sed_GetBorderHeight(list_sed) > disp_GetHeight() -1 ) {
			sed_SetHeight(list_sed, disp_GetHeight() - 3);
		}

		sed_SetPosition(list_sed, row, col);

		/* attach a mouse handler to the popup */
		sed_SetMouse(list_sed, sedmou_GreedyTrack);

		/* put up list of choices */
		sed_Repaint(list_sed);

		if (sed_Go(list_sed) != 0) {
			sed_SetCurrRecord(sed, sed_GetCurrMerge(list_sed));
		}
		sed_UpdateCurrField(sed);
		sed_Close(list_sed);
	}

	/* reset baton */
	sed_SetBaton(sed, -1);
}

char *list_GetChoice(list, num, string)
	char *list;
	int num;
	char *string;
/*
	Copies the num'th choice from the list into string.
	Returns string.
	List is of the form:
	"choice 1,choice 2,choice 3"  (commas delimit).

	returns NULL if num is too large.

	Note: assumes that string is at least MAX_CHOICE_LEN + 1 chars long.
*/
{
	int p, q;
	unsigned len;

	/* skip leading DELIMITERS */
	for (p = 0; list[p] == DELIMITER; p++) {
		;
	}

	for (q = p;; p++) {

		/* skip empty entries */
		if ((list[p] == DELIMITER || list[p] == '\0') && num-- <= 0) {

			if ((len = p - q) > MAX_CHOICE_LEN) {
				len = MAX_CHOICE_LEN;
			}
			memmove(string, list + q, len);
			string[len] = '\0';
			break;
		}
		else if (list[p] == DELIMITER) {
			/* point q to beginning of next entry */
			for (q = p + 1; list[q] == DELIMITER; q++, p++) {
				;
			}
			if (list[q] == '\0') {
				return(NULL);
			}
		}
		else if (list[p] == '\0') {
			
			return(NULL);
		}
	}

	return(string);
}
