/*
	fnsfloat.c     	5/2/87

	% sfloat_funcs

	Float editing functions.
	The field variable should be a float *.
	The record length should be greater than 7.
	These funcs edit floats in a string_like fashion.

	C-scape 3.1
	Copyright (c) 1986, 1987, 1988 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 5/12/88 jmd	added calls to sed_GetScratchPad()
	 9/15/88 jmd 	removed vid_Cursor calls
	 9/17/88 jmd 	added std_ funcs
	10/06/88 jmd	added snum_fenter, added validation, use %g now
	10/09/88 jmd 	added SED_ABORT support
	10/14/88 jdc	added var_size element to field_funcs_struct
	12/16/88 jmd	added vsfloat_fexit

	 6/03/89 jmd	added strclip to senter func
*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "cscape.h"
#include "fnfunc.h"			/* for field functions */
#include "strdecl.h"		/* for C-scape string functions */
#include "scancode.h"

OGLOBAL field_funcs_struct sfloat_funcs = {
	stdBigCur_fenter,
	sfloat_fexit,
	sdouble_fkey,
	sfloat_senter,
	sfloat_sexit,
	sizeof(float)
};

boolean sfloat_fexit(sed)
	sed_type sed;
{
	float val;

	if (sed_GetBaton(sed) != SED_ABORT) {
		sscanf(sed_GetCurrRecord(sed), "%g", &val);
		if ( !valid_Float(val, (char *) sed_GetCurrFieldData(sed, 1)) ) {
			tone();
			sed_BorderPrompt(sed, fnnum_errmsg);
			/* wait for a keystroke */
			while (!kb_Check())
				;
			sed_BorderPrompt(sed, sed_GetCurrFieldData(sed, 0));
			return(FALSE);
		}

		sed_DoFieldSexit(sed, sed_GetFieldNo(sed));
		sed_DoFieldSenter(sed, sed_GetFieldNo(sed));
		sed_UpdateCurrField(sed);
	}

	return(std_fexit(sed));
}

void sfloat_senter(sed, fieldno)
	sed_type sed;
	int fieldno;
/*
	Convert the native float into the record string.
*/
{
	sprintf(sed_GetScratchPad(sed), "%g", *((float *) sed_GetVar(sed, fieldno)));
	strleft(sed_GetScratchPad(sed), sed_GetRecordLen(sed, fieldno));
	strclip(sed_GetScratchPad(sed));
	sed_SetRecord(sed, sed_GetScratchPad(sed), fieldno);

	std_senter(sed, fieldno);
}

void sfloat_sexit(sed, fieldno)
	sed_type sed;
	int fieldno;
/*
	Convert the record string back into the native float.
	Remove commas from string first.
*/
{
	if (sed_GetBaton(sed) != SED_ABORT) {
		strcpy(sed_GetScratchPad(sed), sed_GetRecord(sed, fieldno));
		strnocomma(sed_GetScratchPad(sed));
		sscanf(sed_GetScratchPad(sed), "%g", (float *) sed_GetVar(sed, fieldno));
	}
}


